require "uri"
require "net/http"
module FguSyncPlugin
  def self.init
    @client = TinyTds::Client.new username: 'rulweb', password: 'Rul@0726', host: '192.168.164.12', port: 1433, tds_version: '7.0'
    @client.execute("USE [Personnel]").to_a
    @default_order_fields = {}
    @default_order = 'ASC' # 原本同步庫是 'DESC'
    @debug_sql_mode = false # true will print sql command
    @debug_mode = true # true will print rss_id if it is new record
    @test_mode = false
  end
  def self.close
    @client.close
  end
  def self.localize_data(d)
    ["zh_tw", "en"].map{|l| [l, d]}.to_h
  end
  def self.parse_date(d)
    (d.present? ? (Date.parse(d) rescue nil) : nil)
  end
  def self.camelize(str)
    str.split('_').map{|v| v.capitalize}.join(' ')
  end
  def self.fetch_key(data,key,role=nil,value_flag=false)
    begin
      attr_fields = (value_flag ? role.attribute_fields : data)
      case key
      when 'f_profession_list'
        tmp_key = 'Research Expertise'
      else
        tmp_key = self.camelize(key)
      end
      tmp = attr_fields.select{|v| v.title_translations[:en].upcase.include?(tmp_key.upcase)}.first
      value_flag ? data.where(:attribute_field_id => tmp.id).first : tmp  
    rescue => e
      nil
    end
  end

  def self.sync_journal_papers(staff_id,mp)
    
    puts "*********************************************"
    puts "Syncing journals for #{staff_id}"
    data = self.get_journal_papers(staff_id)
    I18n.locale = :zh_tw
    if data.count > 0
      puts "Data found."
      JournalPaper.where(:member_profile_id => mp.id, :rss2_id.nin => ([nil,''] + data.map{|dt| dt['journal_id']})).destroy_all
      data.each_with_index do |dt, idx|
        rss2id = dt["journal_id"]
       
        pd = self.parse_date(dt["publication_date"])
        paper_title = self.localize_data(dt["paper_title"])
        journal_title = self.localize_data(dt["journal_title"])
        from_to = dt["from_to"].split("-")
        from_to[1] = "" if !from_to[1].present? 
        all_authors = [dt["author1"],dt["author2"],dt["author3"],dt["author4"]]
        all_authors.delete("")
        authors = self.localize_data(all_authors.join(", "))
        note = dt["department"] + " " + dt["country"] + " " + dt["note"]

        if !dt["paper_level"].empty?
          level = JournalLevel.where(:title => dt["paper_level"]).first rescue nil
          if level.nil?
            level = JournalLevel.create(:key => "key_#{idx}", :title_translations => self.localize_data(dt["paper_level"]))
          end
        end

        if !dt["paper_type"].empty?
          type = JournalPaperType.where(:title => dt["paper_type"]).first rescue nil
          if type.nil?
            type = JournalPaperType.create(:key => "key_#{idx}", :title_translations => {"en" => dt["paper_type"], "zh_tw" => dt["paper_type"]})
          end
        end

        jp = JournalPaper.where(:rss2_id => rss2id).first rescue nil
        jp_data = {
          :publication_date => pd, :rss2_id => rss2id, :year => (pd ? pd.strftime("%Y") : ""),
          :authors_translations =>  authors, :paper_title_translations => paper_title,
          :journal_title_translations => journal_title, :vol_no => dt["vol_no"], :issue_no => dt["issue_no"],
          :note => note, :form_to_start => from_to[0], :form_to_end => from_to[1]
        }
        if jp.nil?
          jp = JournalPaper.new(jp_data)
          puts "Saving new journal."
        else
          jp.update_attributes(jp_data)
          puts "Updating journal #{jp.id}."
        end
        jp_mps = MemberProfile.where("tmp_name.zh_tw" => {"$in" => all_authors}).to_a
        jp.member_profiles = jp_mps.size==0 ? [mp] : jp_mps
        jp.journal_levels << level if !level.nil?
        jp.journal_paper_type = type if !type.nil?
        jp.save
      end
    else
      puts "No data found."
    end
    puts "Sync complete for #{staff_id}"
    puts "*********************************************"
  end

  def self.sync_conference_papers(staff_id,mp)
    puts "*********************************************"
    puts "Syncing conference for #{staff_id}"
    data = self.get_conference_papers(staff_id)
    I18n.locale = :zh_tw
    if data.count > 0
      puts "Data found."
      WritingConference.where(:member_profile_id => mp.id, :rss2_id.nin => ([nil,''] + data.map{|dt| dt["conf_id"]})).destroy
      data.each_with_index do |dt, idx|
        rss2id = dt["conf_id"]
       
        start_date = self.parse_date(dt["start_date"])
        end_date = self.parse_date(dt["end_date"])
        paper_title = self.localize_data(dt["paper_title"])
        conference_title = self.localize_data(dt["conference_name"])
        authors = [dt["author1"],dt["author2"],dt["author3"],dt["author4"]]
        authors.delete("")
        authors = self.localize_data(authors.join(", "))
        note = dt["department"] + " " + dt["note"]

        cp = WritingConference.where(:rss2_id => rss2id).first rescue nil
        cp_data = {:period_start_date => start_date, :period_end_date => end_date, :rss2_id => rss2id, :year => (start_date ? start_date.strftime("%Y") : ""), :authors_translations =>  authors, :paper_title_translations => paper_title, :conference_title_translations => conference_title, :note => note, :location_translations => self.localize_data(dt["location"])}
        if cp.nil?
          cp = WritingConference.new(cp_data)
          puts "Saving new conference."
        else
          cp.update_attributes(cp_data)
          puts "Updating conference #{cp.id}."
        end
        cp.member_profile = mp
        cp.save
      end
    else
      puts "No data found."
    end
    puts "Sync complete for #{staff_id}"
    puts "*********************************************"
  end

  def self.sync_books(staff_id,mp)
    puts "*********************************************"
    puts "Syncing book for #{staff_id}"
    data = self.get_books(staff_id)
    I18n.locale = :zh_tw
    if data.count > 0
      puts "Data found."
      Book.where(:member_profile_id => mp.id, :rss2_id.nin => ([nil,''] + data.map{|dt| dt["book_id"]})).destroy
      data.each_with_index do |dt, idx|
        rss2id = dt["book_id"]
       
        pd = self.parse_date(dt["date_of_publication"])
        book_title = self.localize_data(dt["book_title"])
        publisher = self.localize_data(dt["publisher"])
        authors = [dt["author1"],dt["author2"],dt["author3"],dt["author4"]]
        authors.delete("")
        authors = self.localize_data(authors.join(", "))
        note = dt["department"] + " " + dt["location"] + " " + dt["note"]

        if !dt["author_type"].empty?
          authortype = BookAuthorType.where(:title => dt["author_type"]).first rescue nil
          if authortype.nil?
            authortype = BookAuthorType.create(:title_translations => self.localize_data(dt["author_type"]))
          end
        end

        if !dt["book_type"].empty?
          type = BookType.where(:title => dt["book_type"]).first rescue nil
          if type.nil?
            type = BookType.create(:title_translations => self.localize_data(dt["book_type"]))
          end
        end

        book = Book.where(:rss2_id => rss2id).first rescue nil
        book_data = {:publish_date => pd, :rss2_id => rss2id, :year => (pd ? pd.strftime("%Y") : ""), :authors_translations =>  authors, :publisher_translations => publisher, :book_title_translations => book_title, :isbn => dt["isbn"], :note => note, :language => dt["language"], :member_profile => mp}
        if book.nil?
          book = Book.new(book_data)
          puts "Saving new book."
        else
          book.update_attributes(book_data)
          puts "Updating book #{book.id}."
        end
        book.book_author_types << authortype if !authortype.nil?
        book.book_type = type if !type.nil?
        book.member_profile = mp
        book.save
      end
    else
      puts "No data found."
    end
    puts "Sync complete for #{staff_id}"
    puts "*********************************************"
  end

  def self.sync_diplomas(staff_id,mp)
    puts "*********************************************"
    puts "Syncing diplomas for #{staff_id}"
    data = self.get_diplomas(staff_id)
    I18n.locale = :zh_tw
    if data.count > 0
      puts "Data found."
      puts "Deleting old data"
      Diploma.where(:member_profile_id => mp.id).destroy
      data.each_with_index do |dt, idx|
        start_date = self.parse_date(dt["start_date"])
        end_date = self.parse_date(dt["end_date"])
        school_name = self.localize_data(dt["school_name"])
        department = self.localize_data(dt["department"])
        degree = self.localize_data(dt["degree"])
        Diploma.create(:start_date => start_date, :end_date => end_date, :school_name_translations => school_name, :department_translations => department, :degree_translations => degree, :member_profile => mp)
        puts "Diploma created."
      end
    end
    puts "Syncing diplomas for #{staff_id} complete."
    puts "*********************************************"
  end

  def self.sync_experiences(staff_id,mp)
    puts "*********************************************"
    puts "Syncing experiences for #{staff_id}"
    data = self.get_experiences(staff_id)
    I18n.locale = :zh_tw
    if data.count > 0
      puts "Data found."
      puts "Deleting old data"
      Experience.where(:member_profile_id => mp.id, :rss2_id.nin => ([nil,''] + data.map{|dt| dt["rss2_id"]})).destroy
      data.each_with_index do |dt, idx|
        rss2_id = dt['rss2_id']
        start_date = self.parse_date(dt["start_date"])
        end_date = self.parse_date(dt["end_date"])
        organization_title = self.localize_data(dt["organization_title"])
        job_title = self.localize_data(dt["job_title"])
        if !dt["emp_type"].blank?
          type = ExperienceType.where(:title => dt["emp_type"]).first rescue nil
          if type.nil?
            type = ExperienceType.create(:title_translations => self.localize_data(dt["emp_type"]))
          end
        end
        exp = Experience.where(:rss2_id => rss2_id).first
        exp_data = {:rss2_id => rss2_id, :start_date => start_date, :end_date => end_date, :organizationt_title_translations => organization_title, :job_title_translations => job_title, :member_profile => mp, :experience_type => type}
        if exp.nil?
          exp = Experience.create(exp_data)
          puts "Saving new Exp."
        else
          exp.update_attributes(exp_data)
          puts "Updating Exp #{exp.id}."
        end
      end
    end
    puts "Syncing experiences for #{staff_id} complete."
    puts "*********************************************"
  end

  def self.sync_certificates(staff_id,mp)
    puts "*********************************************"
    puts "Syncing certificates for #{staff_id}"
    data = self.get_certificates(staff_id)
    I18n.locale = :zh_tw
    if data.count > 0
      puts "Data found."
      puts "Deleting old data"
      Certificate.where(:member_profile_id => mp.id, :rss2_id.nin => ([nil,''] + data.map{|dt| dt["rss2_id"]})).destroy
      data.each_with_index do |dt, idx|
        rss2_id = dt['rss2_id']
        issue_date = self.parse_date(dt["issue_date"])
        title_translations = self.localize_data(dt["title"])
        issued_by_translations = self.localize_data(dt["issued_by"])
        expiration_date = self.parse_date(dt["expiration_date"])
        year = (issue_date ? issue_date.strftime("%Y") : "")
        if !dt["certificate_type"].blank?
          type = CertificateCategory.where(:certificate_type => dt["certificate_type"]).first rescue nil
          if type.nil?
            type = CertificateCategory.create(:certificate_type_translations => self.localize_data(dt["certificate_type"]))
          end
        end
        cert = Certificate.where(:rss2_id => rss2_id).first
        cert_data = {:rss2_id => rss2_id, :issue_date => issue_date, :title_translations=> title_translations, :issued_by_translations => issued_by_translations,:expiration_date => expiration_date, :year => year, :member_profile => mp, :certificate_category => type}
        if cert.nil?
          cert = Certificate.create(cert_data)
          puts "Saving new Certificate."
        else
          cert.update_attributes(cert_data)
          puts "Updating Certificate #{cert.id}."
        end
      end
    end
    puts "Syncing certificates for #{staff_id} complete."
    puts "*********************************************"
  end

  def self.sync_projects(staff_id,mp)
    puts "*********************************************"
    puts "Syncing projects for #{staff_id}"
    data = self.get_projects(staff_id)
    I18n.locale = :zh_tw
    if data.count > 0
      puts "Data found."
      puts "Deleting old data"
      Project.where(:member_profile_id => mp.id, :rss2_id.nin => ([nil,''] + data.map{|dt| dt["rss2_id"]})).destroy
      data.each_with_index do |dt, idx|
        rss2_id = dt["rss2_id"]
        project_title = self.localize_data(dt["project_title"])
        project_number = dt["project_number"]
        period_start_date = self.parse_date(dt["period_start_date"])
        period_end_date = self.parse_date(dt["period_end_date"])
        unit = self.localize_data(dt["unit"])
        jobs = self.localize_data(dt["jobs"])
        participants = self.localize_data(dt["participants"])
        year = (period_start_date ? period_start_date.strftime("%Y") : "")
        prj = Project.where(:rss2_id => rss2_id).first
        prj_data = {:rss2_id => rss2_id,:project_title_translations => project_title, :project_number=> project_number, :period_start_date => period_start_date, :period_end_date => period_end_date, :unit_translations => unit, :jobs_translations => jobs, :participants_translations => participants, :year => year, :member_profile => mp}
        if prj.nil?
          prj = Project.create(prj_data)
          puts "Saving new Project."
        else
          prj.update_attributes(prj_data)
          puts "Updating Project #{prj.id}."
        end
        puts "Project created."
      end
    end
    puts "Syncing projects for #{staff_id} complete."
    puts "*********************************************"
  end

  def self.sync_honors(staff_id,mp)
    puts "*********************************************"
    puts "Syncing honors for #{staff_id}"
    data = self.get_honors(staff_id)
    I18n.locale = :zh_tw
    if data.count > 0
      puts "Data found."
      puts "Deleting old data"
      Honor.where(:member_profile_id => mp.id, :rss2_id.nin => ([nil,''] + data.map{|dt| dt["rss2_id"]})).destroy
      Honor.where(:member_profile_id => mp.id, :award_name.in => data.map{|dt| dt["award_name"]}.select{|s| s.present?}).destroy
      data.each_with_index do |dt, idx|
        rss2_id = dt["rss2_id"]
        award_date = self.parse_date(dt["award_date"])
        award_name_translations = self.localize_data(dt["award_name"])
        country_translations = self.localize_data(dt["country"])
        ranking_translations = self.localize_data(dt["ranking"])
        honor_type = dt["honor_type"]
        awarding_body_translations = self.localize_data(dt["awarding_body"])
        awarding_unit_translations = self.localize_data(dt["awarding_unit"])
        year = (award_date ? award_date.strftime("%Y") : "")
        if !dt["honor_type"].blank?
          type = HonorType.where(:title => dt["honor_type"]).first rescue nil
          if type.nil?
            type = HonorType.create(:title_translations => self.localize_data(dt["honor_type"]))
          end
        end
        hon = Honor.where(:rss2_id => rss2_id).first
        hon_data = {:rss2_id=> rss2_id, :award_date => award_date, :award_name_translations=> award_name_translations, :country_translations => country_translations, :ranking_translations => ranking_translations, :awarding_body_translations => awarding_body_translations, :awarding_unit_translations => awarding_unit_translations, :year => year, :member_profile => mp, :honor_type => type}
        if hon.nil?
          hon = Honor.create(hon_data)
          puts "Saving new Honor."
        else
          hon.update_attributes(hon_data)
          puts "Updating Honor #{hon.id}."
        end
        puts "Honor created."
      end
    end
    puts "Syncing honors for #{staff_id} complete."
    puts "*********************************************"
  end

  def self.sync_patents(staff_id,mp)
    puts "*********************************************"
    puts "Syncing patents for #{staff_id}"
    data = self.get_patents(staff_id)
    I18n.locale = :zh_tw
    if data.count > 0
      puts "Data found."
      puts "Deleting old data"
      Patent.where(:member_profile_id => mp.id, :rss2_id.nin => ([nil,''] + data.map{|dt| dt["rss2_id"]})).destroy
      data.each_with_index do |dt, idx|
        rss2_id = dt['rss2_id']
        patent_title = self.localize_data(dt["patent_title"])
        patent_country = self.localize_data(dt["patent_country"])
        publication_date = self.parse_date(dt["publication_date"])
        expiration_date = self.parse_date(dt["expiration_date"])
        if !dt["patent_type"].blank?
          type = PatentType.where(:title => dt["patent_type"]).first rescue nil
          if type.nil?
            type = PatentType.create(:title_translations => self.localize_data(dt["patent_type"]))
          end
        end
        year = (publication_date ? publication_date.strftime("%Y") : "")
        pat = Patent.where(:rss2_id => rss2_id).first
        pat_data = {:rss2_id => rss2_id, :patent_title_translations => patent_title, :patent_country_translations => patent_country, :publication_date => publication_date, :expiration_date => expiration_date, :year => year, :member_profile => mp, :patent_type => type}
        if pat.nil?
          pat = Patent.create(pat_data)
          puts "Saving new Patent."
        else
          pat.update_attributes(pat_data)
          puts "Updating Patent #{pat.id}."
        end
      end
    end
    puts "Syncing patents for #{staff_id} complete."
    puts "*********************************************"
  end

  def self.sync_activities(staff_id,mp)
    puts "*********************************************"
    puts "Syncing activities for #{staff_id}"
    data = self.get_activities(staff_id)
    I18n.locale = :zh_tw
    if data.count > 0
      puts "Data found."
      puts "Deleting old data"
      Activity.where(:member_profile_id => mp.id, :rss2_id.nin => ([nil,''] + data.map{|dt| dt["rss2_id"]})).destroy
      data.each_with_index do |dt, idx|
        rss2_id = dt['rss2_id']
        publish_date = self.parse_date(dt["publish_date"])
        updated_at = self.parse_date(dt["updated_at"])
        start_date = self.parse_date(dt["start_date"])
        end_date = self.parse_date(dt["end_date"])
        activity_name = self.localize_data(dt["activity_name"])
        work_title = self.localize_data(dt["work_title"])
        activity_organizer = self.localize_data(dt["activity_organizer"])
        activity_co_organizer = self.localize_data(dt["activity_co_organizer"])
        region = self.localize_data(dt["region"])
        road_show = self.localize_data(dt["road_show"])
        tour_details = self.localize_data(dt["tour_details"])
        award_name = self.localize_data(dt["award_name"])
        awarding_unit = self.localize_data(dt["awarding_unit"])
        authors_type = self.localize_data(dt["authors_type"])
        authors = self.localize_data(dt["authors"])
        year = (publish_date ? publish_date.strftime("%Y") : "")
        act = Activity.where(:rss2_id => rss2_id).first
        act_data = {:rss2_id => rss2_id, :publish_date => publish_date, :updated_at => updated_at, :start_date => start_date, :end_date => end_date, :activity_name_translations => activity_name, :work_title_translations => work_title, :activity_organizer_translations => activity_organizer, :activity_co_organizer_translations => activity_co_organizer, :region_translations => region, :road_show_translations => road_show, :tour_details_translations => tour_details, :award_name_translations => award_name, :awarding_unit_translations => awarding_unit, :authors_type_translations => authors_type, :authors_translations => authors, :year => year, :member_profile => mp}
        if act.nil?
          act = Activity.create(act_data)
          puts "Saving new Activity."
        else
          act.update_attributes(act_data)
          puts "Updating Activity #{act.id}."
        end
      end
    end
    puts "Syncing activities for #{staff_id} complete."
    puts "*********************************************"
  end

  def self.sync_fgu_profile(staff_id, mp)
    puts "*********************************************"
    puts "Syncing profile for #{staff_id}"
    params_to_send = {"plugin" => "profile", "id" => staff_id}
    teacher_data = self.get_teacher_data(staff_id)
    I18n.locale = :zh_tw
    if teacher_data.present?
      all_role_ids = mp.role_ids
      all_role_status_ids = mp.role_status_ids

      case teacher_data["role"]
      when "職員"
        role = Role.where(:key => "staff").first
      when "教師"
        role = Role.where(:key => "teacher").first
      when nil
        role = Role.where(:key => "teacher").first
      end
      if !role.nil?
        puts "Assigning role #{role.key}"
        case teacher_data["job_status"]
        when "專任"
          role_status = role.role_statuses.select{|v| v.title_translations[:en].upcase == "Full-Time".upcase}.first
        when "兼任"
          role_status = role.role_statuses.select{|v| v.title_translations[:en].upcase == "Part-Time".upcase}.first
        else
          role_status = nil
        end
        if role.key == "teacher"
          puts teacher_data
          af = fetch_key(role.attribute_fields,"job_title")
          if !af.nil?
            options = af.option_list.values
            job_title = options.index(options.find{|x| x["zh_tw"] == teacher_data["job_title"]})
            puts job_title.to_s
            if job_title.blank?
              job_title = af.option_list.keys.last.to_i + 1
              af.option_list[job_title] = self.localize_data(teacher_data["job_title"])
              puts af.option_list.to_s
              af.save
            else
              job_title = af.option_list.keys[job_title].to_i
            end
          end
          #saving job title
          av = fetch_key(mp.attribute_values,"job_title",role,true)
          if av.nil?
            field = fetch_key(role.attribute_fields,"job_title")
            if !field.nil?
              av = AttributeValue.new(:attribute_field_id => field.id, :member_profile_id => mp.id, :key => "job_title")
            end
          end
          if !av.nil?
            av.value= job_title
            av.save
          end
          #saving profession list
          av = fetch_key(mp.attribute_values,"f_profession_list",role,true)
          if av.nil?
            field = fetch_key(role.attribute_fields,"f_profession_list")
            if !field.nil?
              av = AttributeValue.new(:attribute_field_id => field.id, :member_profile_id => mp.id, :key => "f_profession_list")
            end
          end
          if !av.nil?
            av.value= self.localize_data(teacher_data["profession_list"])
            av.save
          end
          #saving courses
          av = fetch_key(mp.attribute_values,"current_course",role,true)
          if av.nil?
            field = fetch_key(role.attribute_fields,"current_course")
            if !field.nil?
              av = AttributeValue.new(:attribute_field_id => field.id, :member_profile_id => mp.id)
            end
          end
          if !av.nil?
            course_html = ""
            if teacher_data["courses"].blank?
              av.value= self.localize_data(course_html)
              av.save
            end
            teacher_data["courses"].each do |course|
              course_url = course["course_url"]
              course_url.gsub!(/http:\/\/(.*\.fgu\.edu\.tw)/, 'https://\1')
              course_html += "<a data-year='#{course["year"].to_i + 1911}' data-sem='#{course["sem"]}' href='#{course_url}' target='_blank'>#{course["course_id"]} #{course["course_name"]}</a><br />"
              av.value = self.localize_data(course_html)
              av.save
            end
          end
        end
      end
      mp.email = teacher_data["email"]
      mp.office_tel = teacher_data["office_tel"]
      mp.address_translations = self.localize_data(teacher_data["address"])
      mp.first_name_translations = self.localize_data(teacher_data["name"])
      if (!role.nil? && !all_role_ids.include?(role.id))
        all_role_ids << role.id
      end
      if (!role_status.nil? && !all_role_status_ids.include?(role_status.id))
        all_role_status_ids << role_status.id
      end

      puts [staff_id,all_role_ids.inspect]
      if !all_role_ids.blank?
        mp.role_ids = all_role_ids
        all_role_ids.each do |r_id| 
          rr = Role.find(r_id)
          if !rr.member_profile_ids.include?(mp.id)
            rr.member_profile_ids << mp.id
            rr.save
          end
        end
        puts 'modify roles'
      end
      if !all_role_status_ids.blank?
        mp.role_status_ids = all_role_status_ids
        all_role_status_ids.each do |r_id| 
          rr = RoleStatus.find(r_id)
          if !rr.member_profile_ids.include?(mp.id)
            rr.member_profile_ids << mp.id
            rr.save
          end
        end
        puts 'modify role_statuses'
      end
      mp.save
    end
    puts "Syncing profile for #{staff_id} complete."
    puts "*********************************************"
  end

  def self.get_sql(table_name, filter_params={}, order_fields=nil)
    key_mapping = {
      'v_rul_research_01' => {
        "紀錄編號" => "journal_id",
        "結束日" => "publication_date",
        "填報系所" => "department",
        "合著者1姓名" => "author1",
        "合著者2姓名" => "author2",
        "合著者3姓名" => "author3",
        "合著者4以上" => "author4",
        "期刊_學報之論文名稱" => "paper_title",
        "期刊_學報名稱" => "journal_title",
        "期刊_學報卷數" => "vol_no",
        "期刊_學報期數" => "issue_no",
        "論文發表型式" => "paper_type",
        "國別地區代號" => "country",
        "期刊_學報頁碼" => "from_to",
        "期刊資料庫" => "paper_level",
        "備註" => "note"
      },
      'v_rul_research_02' => {
        "紀錄編號" => "conf_id",
        "開始日" => "start_date",
        "結束日" => "end_date",
        "填報系所" => "department",
        "合著者1姓名" => "author1",
        "合著者2姓名" => "author2",
        "合著者3姓名" => "author3",
        "合著者4以上" => "author4",
        "論文名稱" => "paper_title",
        "會議名稱" => "conference_name",
        "出版_發表地區" => "location",
        "備註" => "note"
      },
      'v_rul_research_03' => {
        "紀錄編號" => "book_id",
        "結束日" => "date_of_publication",
        "填報部門" => "department",
        "作者類型" => "author_type",
        "專書類別" => "book_type",
        "合著者1姓名" => "author1",
        "合著者2姓名" => "author2",
        "合著者3姓名" => "author3",
        "合著者4以上" => "author4",
        "專書名稱" => "book_title",
        "使用語文" => "language",
        "出版社_發表處所名稱" => "publisher",
        "專書ISBN編號或無" => "isbn",
        "出版_發表地區" => "location",
        "備註" => "note"
      },
      'vrul_sch' => {
        "f_start" => "start_date",
        "f_end" => "end_date",
        "f_school" => "school_name",
        "f_dep" => "department",
        "f_degree" => "degree"
      },
      'vrul_exp' => {
        "f_start" => "start_date",
        "f_end" => "end_date",
        "f_comp" => "organization_title",
        "f_empname" => "job_title"
      },
      'vrul_inemp' => {
        "f_start" => "start_date",
        "f_end" => "end_date",
        "f_unit" => "organization_title",
        "f_empname" => "job_title",
        "f_emptype" => "emp_type"
      },
      'dbo.v_rul_research_credential' => {
        "紀錄編號" => "rss2_id",
        "核發日期" => "issue_date",
        "證照名稱" => "title",
        "發照機關" => "issued_by",
        "證照類別" => "certificate_type",
        "有效期限" => "expiration_date"
      },
      'dbo.v_rul_research_award' => {
        "紀錄編號" => "rss2_id",
        "獲獎日期" => "award_date",
        "獎項名稱" => "award_name",
        "國別/地區" => "country",
        "名次" => "ranking",
        "區域" => "honor_type",
        "頒發機構名稱" => "awarding_body",
        "頒獎單位名稱"=> "awarding_unit"
      },
      'dbo.v_rul_research_patent' => {
        "紀錄編號" => "rss2_id",
        "專利名稱" => "patent_title",
        "申請國別" => "patent_country",
        "生效日期"=> "publication_date",
        "有效期限" => "expiration_date",
        "專利類型" => "patent_type"
      },
      'dbo.v_rul_research_create' => {
        "紀錄編號" => "rss2_id",
        "首次發表時間" => "publish_date",
        "更新日期" => "updated_at",
        "展演舉行起日" => "start_date",
        "展演舉行迄日" => "end_date",
        "展演活動名稱" => "activity_name",
        "展演作品名稱" => "work_title",
        "展演主辦單位全銜" => "activity_organizer",
        "展演協辦單位全銜" => "activity_co_organizer",
        "展演活動辦理國別/地區" => "region",
        "是否為巡演" => "road_show",
        "巡演明細" => "tour_details",
        "獎名" => "award_name",
        "頒獎單位" => "awarding_unit",
        "作者別" => "authors_type",
        "所有作者群" => "authors"
      },
      'dbo.v_rul_research_plan' => {
        "紀錄編號" => "rss2_id",
        "計畫名稱" => "project_title",
        "校外計畫編號" => "project_number",
        "計畫起日期" => "period_start_date",
        "計畫迄日期" => "period_end_date",
        "經費補助單位(補助單位)" => "unit",
        "填報者類型(職稱/擔任之工作)" => "jobs",
        "所有參與者" => "participants"
      },
      'vrul_base' => {
        "f_email" => "email",
        "f_phone" => "office_tel",
        "f_office" => "address",
        "f_name" => "name",
        "f_teachwork" => "role",
        "f_empname" => "job_title",
        "f_emptype" => "job_status"
      },
      'vrul_profession' => {
        'f_profession_list' => 'profession_list'
      },
      'vrul_nowteachcour' => {
        "f_ayear" => "year",
        "f_asem" => "sem",
        "f_cour_id" => "course_id",
        "f_cour_name" => "course_name",
        "f_cour_url" => "course_url"
      }
    }
    if key_mapping.has_key?(table_name) && key_mapping[table_name].present?
      tmp = key_mapping[table_name]
      mapping_text = tmp.map{|k,v| "[#{k}] as '#{v}'"}.join(', ')
    else
      mapping_text = '*'
    end
    if order_fields.nil?
      sql = "SELECT #{mapping_text} FROM #{table_name}"
      field_prefix = ""
    else # rss_id is the row index
      if order_fields === false
        sql = "SELECT #{mapping_text} FROM (SELECT ROW_NUMBER() OVER(ORDER BY (SELECT NULL)) "
      else
        sql = "SELECT #{mapping_text} FROM (SELECT ROW_NUMBER() OVER(ORDER BY "
        sql += order_fields.map{|k,v| "#{k} #{v}"}.join(', ')
        sql += ") "
      end
      sql += "AS rss_id,* FROM [#{table_name}]) r"
      field_prefix = "r."
    end
    if filter_params.present?
      sql += " WHERE "
      filter_params.each_with_index do |(k,v), i|
        if i != 0
          sql += " AND "
        end
        if v.class == Array
          if v.count > 0
            sql += "("
            v.each_with_index do |vv, i|
              if i != 0
                sql += " OR "
              end
              if vv.class == String
                sql += "#{field_prefix}#{k} = \"#{vv}\""
              else
                sql += "#{field_prefix}#{k} = #{vv}"
              end
            end
            sql += ")"
          end
        else
          if v.class == String
            sql += "#{field_prefix}#{k} = \"#{v}\""
          else
            sql += "#{field_prefix}#{k} = #{v}"
          end
        end
      end
    end
    if @debug_sql_mode
      puts sql
    end
    sql
  end

  def self.query_db(table_name, filter_params={})
    order_fields = {} # true => order by first column with desc order
    rss_id_offsets = {}
    # rss_id_offsets = {
    #   'rsspublish' => 1073443
    # }
    extra_condition = {
      'rss_pastapos_a' => {
        'dsp_flg' => 1,
        'inst_cod' => ['309210000Q','A09520000Q']
      },
      'rss_pastapos_b' => {
        'dsp_flg' => 1
      },
      'rss_paedu_vw' => {
        'dsp_flg' => 1
      }
    }
    order_field = order_fields[table_name]
    if order_field === true
      order_field = @default_order_fields[table_name]
      if order_field.nil?
        order_field = {@client.execute("SELECT name FROM sys.columns WHERE object_id = OBJECT_ID('#{table_name}')").to_a.map{|h| h['name']}.first => @default_order}
        @default_order_fields[table_name] = order_field
      end
    end
    if @test_mode
      filter_params = {}
    end
    #'origin field name' => 'new field name'
    extra_key_mapping = {}
    sql = self.get_sql(table_name, filter_params.merge(extra_condition[table_name].to_h), order_field)
    result = @client.execute(sql).to_a
    if extra_key_mapping.has_key?(table_name) && extra_key_mapping[table_name].present?
      tmp = extra_key_mapping[table_name]
      rss_id_offset = rss_id_offsets[table_name]
      result.each do |h|
        if rss_id_offset && h['rss_id']
          h['rss_id'] = h['rss_id'] + rss_id_offset
        end
        h.each do |k,v|
          if v.class == String
            h[k] = v.strip
          end
        end
        tmp.each do |k,v|
          if k.class != String
            tmp_value = k
          else
            tmp_value = h[k]
          end
          if v.class == Array
            include_key = false
            v.each do |vv|
              if vv.include?('.')
                vv_k, locale = vv.split('.')
                include_key = true if vv_k == k
                unless h.has_key?(vv_k)
                  h[vv_k] = {}
                end
                h[vv_k][locale] = tmp_value
              else
                include_key = true if vv == k
                h[vv] = tmp_value
              end
            end
            unless include_key
              h.delete(k)
            end
          else
            if v.include?('.')
              v_k, locale = v.split('.')
              unless h.has_key?(v_k)
                h[v_k] = {}
              end
              h[v_k][locale] = tmp_value
            else
              if v.ends_with?('_date') && tmp_value.present? && tmp_value.scan(/\d/).count == 6
                if tmp_value.length == 6 #年月,ex: 202005
                  tmp_value += "01"
                else #有分隔符號
                  tmp_value = "#{tmp_value.gsub(/\/|-|,/, '-')}-01"
                end
              end
              h[v] = tmp_value
            end
            h.delete(k)
          end
        end
      end
    else
      result.each do |h|
        if rss_id_offset && h['rss_id']
          h['rss_id'] = h['rss_id'] + rss_id_offset
        end
        h.each do |k,v|
          if v.class == String
            h[k] = v.strip
          end
        end
      end
    end
    if @test_mode && table_name.start_with?('rss')
      print result[0]
      puts nil
      raise 'error'
    end
    result
  end

  def self.get_journal_papers(staff_id)
    result = query_db('v_rul_research_01', {"填報者公號"=> "F#{staff_id}"})
  end

  def self.get_conference_papers(staff_id)
    result = query_db('v_rul_research_02', {"填報者公號"=> "F#{staff_id}"})
  end

  def self.get_research_papers(staff_id)
    result = query_db('rsspublish', {:sta_num=>staff_id, :publ_tpe=>'05'})
  end

  def self.get_books(staff_id)
    result = query_db('v_rul_research_03', {"填報者公號"=> "F#{staff_id}"})
  end

  def self.get_diplomas(staff_id)
    result = query_db('vrul_sch', {:f_uni_id=> "F#{staff_id}"})
  end

  def self.get_experiences(staff_id)
    data = query_db('vrul_exp', {:f_uni_id=> "F#{staff_id}"})
    data1 = query_db('vrul_inemp', {:f_uni_id=> "F#{staff_id}"})
    data + data1
  end

  def self.get_certificates(staff_id)
    result = query_db('dbo.v_rul_research_credential', {"填報者公號"=> "F#{staff_id}"})
  end

  def self.get_activities(staff_id)
    result = query_db('dbo.v_rul_research_create', {"填報者公號"=> "F#{staff_id}"})
  end
  def self.get_projects(staff_id)
    result = query_db('dbo.v_rul_research_plan', {"填報者公號"=> "F#{staff_id}"})
  end

  def self.get_honors(staff_id)
    result = query_db('dbo.v_rul_research_award', {"填報者公號"=> "F#{staff_id}"})
  end

  def self.get_patents(staff_id)
    result = query_db('dbo.v_rul_research_patent', {"填報者公號"=> "F#{staff_id}"})
  end

  def self.get_teacher_data(staff_id)
    data = query_db('vrul_base', {:f_uni_id=> "F#{staff_id}"})
    data1 = query_db('vrul_profession', {:f_uni_id=> "F#{staff_id}"})
    tmp = {}
    tmp.merge!(data[0].to_h).merge!(data1[0].to_h)
    tmp['courses'] = query_db('vrul_nowteachcour', {:f_uni_id=> "F#{staff_id}"})
    tmp
  end

end
namespace :sync_fgu_personal_plugins do
  task :sync,[:uid] => :environment do |task,args|
    #require "#{ENV['PWD']}/app/models/member_profile" #it will cause circular dependency
    FguSyncPlugin.init
    if args.uid.present?
      mps = MemberProfile.where(:uid=>args.uid).to_a
    else
      mps = MemberProfile.all.to_a
    end
    today = Date.today
    mps.each do |mp|
      staff_id = mp.sid rescue nil
      if (!staff_id.blank? rescue false)
        FguSyncPlugin.sync_journal_papers(staff_id,mp)
        FguSyncPlugin.sync_conference_papers(staff_id,mp)
        FguSyncPlugin.sync_books(staff_id,mp)
        FguSyncPlugin.sync_diplomas(staff_id,mp)
        FguSyncPlugin.sync_experiences(staff_id,mp)
        FguSyncPlugin.sync_certificates(staff_id,mp)
        FguSyncPlugin.sync_honors(staff_id,mp)
        FguSyncPlugin.sync_patents(staff_id,mp)
        FguSyncPlugin.sync_projects(staff_id,mp)
        FguSyncPlugin.sync_activities(staff_id, mp)
        FguSyncPlugin.sync_fgu_profile(staff_id, mp)
        
      end
    end
    FguSyncPlugin.close
  end
end